#include <Stepper.h>
#include <DFRobotDFPlayerMini.h>
#include <SoftwareSerial.h>

const int stepsPerRevolution = 2048;  // Number of steps per revolution for motor
Stepper myStepper(stepsPerRevolution, 2, 3, 4, 5); // Pins for ULN2003 driver

const int pirPin = 7;     // PIR sensor pin
const int ledPin = 13;    // LED indicator pin
int pirState = LOW;       // Start with no motion detected
int motorSpeed = 15;      // Speed for the motor (in RPM)
int stepsToRotate = stepsPerRevolution * (10 * motorSpeed / 60); // Rotate for 10 seconds
int stepsToRotateBack = -stepsPerRevolution * (10 * motorSpeed / 60); // Rotate for 10 seconds


unsigned long motionDetectedTime = 0; // Timestamp of motion detected
bool motorSpinning = false;           // Flag to track if the motor is spinning
bool audioPlayed = false;             // Flag to track if the audio has been played after motor stops

// Setup DFPlayer Mini
SoftwareSerial mySerial(10, 11);          // RX, TX pins for DFPlayer
DFRobotDFPlayerMini myDFPlayer;

void setup() {
  pinMode(pirPin, INPUT);
  pinMode(ledPin, OUTPUT);              // Set LED pin as output
  digitalWrite(ledPin, LOW);            // LED off initially
  myStepper.setSpeed(motorSpeed);

  // Initialize DFPlayer Mini
  mySerial.begin(9600);
  if (!myDFPlayer.begin(mySerial)) {
    // If DFPlayer can't initialize, blink LED rapidly as error signal
    while (true) {
      digitalWrite(ledPin, HIGH);
      delay(100);
      digitalWrite(ledPin, LOW);
      delay(100);
    }
  }
  myDFPlayer.setTimeOut(500);          // Set serial communication timeout
  myDFPlayer.volume(30);               // Set volume to 30 (0 to 30 range)
  
  randomSeed(analogRead(0));           // Seed random number generator
}

void loop() {
  int pirValue = digitalRead(pirPin);  // Read PIR sensor

  // If motion is detected and motor is not already spinning
  if (pirValue == HIGH && !motorSpinning) {
    motorSpinning = true;
    audioPlayed = false;               // Reset audio flag for the next cycle
    pirState = HIGH;
    motionDetectedTime = millis();
    digitalWrite(ledPin, HIGH);        // Turn on LED to indicate motion detected
    myStepper.step(stepsToRotate);    //Rotate Stepper
  }

  // Check if motor rotation is complete and audio hasn't been played yet
  if (motorSpinning && !audioPlayed) {
    unsigned long elapsedTime = millis() - motionDetectedTime;

    if (elapsedTime >= 10000) {  // Check if motor rotated for 10 seconds
      // Select a random audio track (1, 2, or 3) and play it
      int trackNumber = random(1, 4); // Generates a random number between 1 and 3
      myDFPlayer.play(trackNumber);
      audioPlayed = true;           // Set flag to indicate audio has been played

      // Start a 20 second delay before motor returns to starting position
      motionDetectedTime = millis();
    }
  }

  // Check if 20 seconds have passed since the motor stopped and audio played
  if (motorSpinning && audioPlayed) {
    unsigned long elapsedTime = millis() - motionDetectedTime;

    if (elapsedTime >= 20000) {
      pirState = LOW;
      motorSpinning = false;

      // Rotate motor counterclockwise to return to starting position
      myStepper.step(stepsToRotateBack);    //Rotate Stepper

      digitalWrite(ledPin, LOW);       // Turn off LED to indicate end of sequence
    }
  }

  delay(500); // Short delay to reduce unnecessary checks
}
